@@ -163,8 +163,16 @@ module Dnsruby
     @@authority_cache = Hash.new
     @@zones_cache = nil
         
-    def initialize(res = Resolver.new)
-      @resolver = res
+    def initialize(res = nil)
+      if (res)
+        @resolver = res
+      else
+        if (defined?@@nameservers && @@nameservers.length > 0)
+          @resolver = Resolver.new({:nameserver => @@nameservers})
+        else
+          @resolver = Resolver.new
+        end
+      end
       @ipv6_ok = false
     end
     #Initialize the hint servers.  Recursive queries need a starting name
@@ -184,17 +192,33 @@ module Dnsruby
     end
     def Recursor.set_hints(hints, resolver)
       TheLog.debug(";; hints(#{hints.inspect})\n")
+      @resolver = resolver
+      if (resolver.single_resolvers.length == 0)
+        resolver = Resolver.new()
+      end
+      if (hints && hints.length > 0)
+        resolver.nameservers=hints
+        if (String === hints)
+          hints = [hints]
+        end
+        hints.each {|hint|
+          @@hints = Hash.new
+          @@hints[hint]=hint
+        }
+      end
       if (!hints && @@nameservers)
         @@hints=(@@nameservers)
       else
         @@nameservers=(hints)
+        @@hints = hints
       end
       TheLog.debug(";; verifying (root) zone...\n")
       # bind always asks one of the hint servers
       # for who it thinks is authoritative for
       # the (root) zone as a sanity check.
       # Nice idea.
-          
+
+      #      if (!@@hints || @@hints.length == 0)
       resolver.recurse=(1)
       packet=resolver.query_no_validation_or_recursion(".", "NS", "IN")
       hints = Hash.new
@@ -210,6 +234,8 @@ module Dnsruby
               hints[server] = AddressCache.new
             end
           end
+          # @TODO@ Some resolvers (e.g. 8.8.8.8) do not send an additional section -
+          # need to make explicit queries for these :(
           packet.additional.each do |rr|
             TheLog.debug(";; ADDITIONAL: "+rr.inspect+"\n")
             server = rr.name.to_s.downcase
@@ -230,7 +256,7 @@ module Dnsruby
                 end
               end
                   
-            end 
+            end
           end
         end
         #                      foreach my $server (keys %hints) {
@@ -245,22 +271,39 @@ module Dnsruby
         @@hints = {}
       end
       if (@@hints.size > 0)
-        if (@debug)
-          TheLog.info(";; USING THE FOLLOWING HINT IPS:\n")
-          @@hints.values.each do |ips|
-            ips.each do |server|
-              TheLog.info(";;  #{server}\n")
-            end
+        TheLog.info(";; USING THE FOLLOWING HINT IPS:\n")
+        @@hints.values.each do |ips|
+          ips.each do |server|
+            TheLog.info(";;  #{server}\n")
           end
         end
       else
-        warn "Server ["+(@@nameservers)[0].to_s+".] did not give answers"
+        raise ResolvError.new( "Server ["+(@@nameservers)[0].to_s+".] did not give answers")
       end
           
       # Disable recursion flag.
       resolver.recurse=(0)
+      #      end
           
       #  return $self->nameservers( map { @{ $_ } } values %{ $self->{'hints'} } );
+      if (Array === @@hints)
+        temp = []
+        @@hints.each {|hint|
+          temp.push(hint)
+        }
+        @@hints = Hash.new
+        count = 0
+        temp.each {|hint|
+          print "Adding hint : #{temp[count]}\n"
+          @@hints[count] = temp[count]
+          count += 1
+        }
+      end
+      if (String === @@hints)
+        temp = @@hints
+        @@hints = Hash.new
+        @@hints[0] = temp
+      end
       @@nameservers = @@hints.values
       return @@nameservers
     end
@@ -293,7 +336,7 @@ module Dnsruby
       @@zones_cache = Hash.new # key zone_name, values Hash of servers and AddressCaches
       @@zones_cache["."] = @@hints
 
-    @@authority_cache = Hash.new
+      @@authority_cache = Hash.new
     end
 
     def query_no_validation_or_recursion(name, type=Types.A, klass=Classes.IN) # :nodoc: all
@@ -358,10 +401,10 @@ module Dnsruby
         zone = nil
         @@mutex.synchronize{
           zone = @@zones_cache[name]
+          if (zone != nil)
+            return name
+          end
         }
-        if (zone != nil)
-          return name
-        end
         return false if name=="." 
         # strip the name up to the first dot
         first_dot = name.index(".")
@@ -398,11 +441,7 @@ module Dnsruby
     end
         
     def _dorecursion(name, type, klass, known_zone, known_authorities, depth, no_validation) # :nodoc:
-      cache = nil
-      @@mutex.synchronize{
-        cache = @@authority_cache # this acts as a store of known server_names - NOT zones
-      }
-          
+
       if ( depth > 255 )
         TheLog.debug(";; _dorecursion() Recursion too deep, aborting...\n")
         @errorstring="Recursion too deep, aborted"
@@ -411,110 +450,110 @@ module Dnsruby
           
       known_zone.sub!(/\.*$/, ".")
           
-      # Get IPs from authorities
       ns = [] # Array of AddressCaches (was array of array of addresses)
-      known_authorities.keys.each do |ns_rec|
-        if (known_authorities[ns_rec] != nil  && known_authorities[ns_rec] != [] )
-          cache[ns_rec] = known_authorities[ns_rec]
-          ns.push(cache[ns_rec])
-        elsif (cache[ns_rec]!=nil && cache[ns_rec]!=[])
-          known_authorities[ns_rec] = cache[ns_rec]
-          ns.push(cache[ns_rec])
-        end
-      end
-          
-      if (ns.length == 0)
-        found_auth = 0
-        #        @@mutex.synchronize { # @TODO@ Lock access to @@known_authorities
-        TheLog.debug(";; _dorecursion() Failed to extract nameserver IPs:")
-        TheLog.debug(known_authorities.inspect + cache.inspect)
+      @@mutex.synchronize{
+        # Get IPs from authorities
         known_authorities.keys.each do |ns_rec|
-          if (known_authorities[ns_rec]==nil || known_authorities[ns_rec]==[])
-            TheLog.debug(";; _dorecursion() Manual lookup for authority [#{ns_rec}]")
+          if (known_authorities[ns_rec] != nil  && known_authorities[ns_rec] != [] )
+            @@authority_cache[ns_rec] = known_authorities[ns_rec]
+            ns.push(@@authority_cache[ns_rec])
+          elsif (@@authority_cache[ns_rec]!=nil && @@authority_cache[ns_rec]!=[])
+            known_authorities[ns_rec] = @@authority_cache[ns_rec]
+            ns.push(@@authority_cache[ns_rec])
+          end
+        end
+          
+        if (ns.length == 0)
+          found_auth = 0
+          TheLog.debug(";; _dorecursion() Failed to extract nameserver IPs:")
+          TheLog.debug(known_authorities.inspect + @@authority_cache.inspect)
+          known_authorities.keys.each do |ns_rec|
+            if (known_authorities[ns_rec]==nil || known_authorities[ns_rec]==[])
+              TheLog.debug(";; _dorecursion() Manual lookup for authority [#{ns_rec}]")
                 
-            auth_packet=nil
-            ans=[]
+              auth_packet=nil
+              ans=[]
                 
-            # Don't query for V6 if its not there.
-            # Do this in parallel
-            ip_mutex = Mutex.new
-            ip6_thread = Thread.start {
-              if ( @ipv6_ok)
-                auth_packet = _dorecursion(ns_rec,"AAAA", klass,  # packet
+              # Don't query for V6 if its not there.
+              # Do this in parallel
+              ip_mutex = Mutex.new
+              ip6_thread = Thread.start {
+                if ( @ipv6_ok)
+                  auth_packet = _dorecursion(ns_rec,"AAAA", klass,  # packet
+                    ".",               # known_zone
+                    @@hints,  # known_authorities
+                    depth+1);         # depth
+                  ip_mutex.synchronize {
+                    ans.push(auth_packet.answer) if auth_packet
+                  }
+                end
+              }
+
+              ip4_thread = Thread.start {
+                auth_packet = _dorecursion(ns_rec,"A",klass,  # packet
                   ".",               # known_zone
                   @@hints,  # known_authorities
                   depth+1);         # depth
+                
                 ip_mutex.synchronize {
-                  ans.push(auth_packet.answer) if auth_packet
+                  ans.push(auth_packet.answer ) if auth_packet
                 }
-              end
-            }
-
-            ip4_thread = Thread.start {
-              auth_packet = _dorecursion(ns_rec,"A",klass,  # packet
-                ".",               # known_zone
-                @@hints,  # known_authorities
-                depth+1);         # depth
-                
-              ip_mutex.synchronize {
-                ans.push(auth_packet.answer ) if auth_packet
               }
-            }
-            ip6_thread.join
-            ip4_thread.join
+              ip6_thread.join
+              ip4_thread.join
                 
-            if ( ans.length > 0 )
-              TheLog.debug(";; _dorecursion() Answers found for [#{ns_rec}]")
-              #          foreach my $rr (@ans) {
-              ans.each do |rr_arr|
-                rr_arr.each do |rr|
-                  TheLog.debug(";; RR:" + rr.inspect + "")
-                  if (rr.type == Types::CNAME)
-                    # Follow CNAME
-                    server = rr.name.to_s.downcase
-                    if (server)
-                      server.sub!(/\.*$/, ".")
-                      if (server == ns_rec)
-                        cname = rr.cname.downcase
-                        cname.sub!(/\.*$/, ".")
-                        TheLog.debug(";; _dorecursion() Following CNAME ns [#{ns_rec}] -> [#{cname}]")
-                        if (!(known_authorities[cname]))
-                          known_authorities[cname] = AddressCache.new
+              if ( ans.length > 0 )
+                TheLog.debug(";; _dorecursion() Answers found for [#{ns_rec}]")
+                #          foreach my $rr (@ans) {
+                ans.each do |rr_arr|
+                  rr_arr.each do |rr|
+                    TheLog.debug(";; RR:" + rr.inspect + "")
+                    if (rr.type == Types::CNAME)
+                      # Follow CNAME
+                      server = rr.name.to_s.downcase
+                      if (server)
+                        server.sub!(/\.*$/, ".")
+                        if (server == ns_rec)
+                          cname = rr.cname.downcase
+                          cname.sub!(/\.*$/, ".")
+                          TheLog.debug(";; _dorecursion() Following CNAME ns [#{ns_rec}] -> [#{cname}]")
+                          if (!(known_authorities[cname]))
+                            known_authorities[cname] = AddressCache.new
+                          end
+                          known_authorities.delete(ns_rec)
+                          next
                         end
-                        known_authorities.delete(ns_rec)
-                        next
                       end
-                    end
-                  elsif (rr.type == Types::A || rr.type == Types::AAAA )
-                    server = rr.name.to_s.downcase
-                    if (server)
-                      server.sub!(/\.*$/, ".")
-                      if (known_authorities[server]!=nil)
-                        ip = rr.address.to_s
-                        TheLog.debug(";; _dorecursion() Found ns: #{server} IN A #{ip}")
-                        cache[server] = known_authorities[server]
-                        cache[ns_rec].push([ip, rr.ttl])
-                        found_auth+=1
-                        next
+                    elsif (rr.type == Types::A || rr.type == Types::AAAA )
+                      server = rr.name.to_s.downcase
+                      if (server)
+                        server.sub!(/\.*$/, ".")
+                        if (known_authorities[server]!=nil)
+                          ip = rr.address.to_s
+                          TheLog.debug(";; _dorecursion() Found ns: #{server} IN A #{ip}")
+                          @@authority_cache[server] = known_authorities[server]
+                          @@authority_cache[ns_rec].push([ip, rr.ttl])
+                          found_auth+=1
+                          next
+                        end
                       end
                     end
+                    TheLog.debug(";; _dorecursion() Ignoring useless answer: " + rr.inspect + "")
                   end
-                  TheLog.debug(";; _dorecursion() Ignoring useless answer: " + rr.inspect + "")
                 end
+              else
+                TheLog.debug(";; _dorecursion() Could not find A records for [#{ns_rec}]")
               end
-            else
-              TheLog.debug(";; _dorecursion() Could not find A records for [#{ns_rec}]")
             end
           end
+          if (found_auth > 0)
+            TheLog.debug(";; _dorecursion() Found #{found_auth} new NS authorities...")
+            return _dorecursion( name, type, klass, known_zone, known_authorities, depth+1)
+          end
+          TheLog.debug(";; _dorecursion() No authority information could be obtained.")
+          return nil
         end
-        if (found_auth > 0)
-          TheLog.debug(";; _dorecursion() Found #{found_auth} new NS authorities...")
-          return _dorecursion( name, type, klass, known_zone, known_authorities, depth+1)
-        end
-        TheLog.debug(";; _dorecursion() No authority information could be obtained.")
-        return nil
-        #        }
-      end
+      }
           
       # Cut the deck of IPs in a random place.
       TheLog.debug(";; _dorecursion() cutting deck of (" + ns.length.to_s + ") authorities...")
@@ -540,6 +579,7 @@ module Dnsruby
         query = Message.new(name, type, klass)
         query.header.rd = false
         query.do_validation = true
+        query.do_caching = false
         query.do_validation = false if no_validation
         #            print "Sending msg from resolver, dnssec = #{resolver.dnssec}, do_validation = #{query.do_validation}\n"
         packet = resolver.send_message(query)
